(ns coderloop.export-files
  (:use clojure.java.io)
  (:use [clojure.contrib [duck-streams :only [file-str]]])
  (:use [rlm
	 [classpath-utils :only [classpath]]
	 [shell-write :only [sw]]])
  (:require clojure.string)
  (:import java.io.File))

(import '[org.apache.commons.io FileUtils])

;;(use '[telmo.rlm [dist :only [dist]]])

(def *nailgun* true)

;;; put standard base-path on the classpath
(def standard-base-directory (file-str "~/.clojure-exports/"))

(defn better-export-files [namespace name]
    (binding [*compile-path* (.getPath standard-base-directory)]
    ;;clear out classes -- prepare temp directory
    (FileUtils/forceMkdir standard-base-directory)
    (FileUtils/deleteDirectory standard-base-directory)
    (FileUtils/forceMkdir standard-base-directory)      
      ;; make classes
    (compile (symbol (str namespace)))
    
    ;; make jar
;;    (jar {:jarfile jar-name :basedir temp-class-dir})
    
    ;; move jar
;;    (FileUtils/copyFileToDirectory jar-name target-lib-dir)
    ))

(defmulti source-location class)

(defmethod source-location clojure.lang.Var [sym]
	   (let [source? (:file (meta sym))]
	     (if source?
	       (.getResource (clojure.lang.RT/baseLoader) source?)
	       nil)))

(defmethod source-location java.lang.Class [sym]
	   (let [source? (.getCodeSource (.getProtectionDomain sym))]
	     (if source?
	       (.getLocation source?)
	       nil)))

(defn all-dependent-namespaces [namespace]
  (set
   (concat
    [namespace]
    
    (vals (ns-aliases namespace))

    (map #(.ns %)
	 (filter #(= (class %) clojure.lang.Var)
		 (concat 
		  
		  (vals (ns-map namespace))
		  (vals (ns-refers namespace))
		  
		  ))))))

(defn deep-dependent-namespaces [namespace-set]
  (let [new-namespaces (set (mapcat all-dependent-namespaces namespace-set))]
    ;;(println (count new-namespaces))
    (if (= new-namespaces namespace-set)
      namespace-set
      (recur new-namespaces))))


(defn dependencies-url
  "returns all of the files necessary to succesfully run the namespace."
  [namespace]
  (set
   (remove nil?
	   (map source-location
		(map second
		     (mapcat ns-map (deep-dependent-namespaces #{namespace})))))))


(defn trans-print [x] (println x) x) 

(defn dependencies-file [namespace]
  (map file-str
       (set
	(map #(.replaceFirst % "file:" "")
	(map  #(if (.contains % ".jar!/")
		 (clojure.string/replace % #"\.jar.*" ".jar")
		 %)
	      (map #(.getFile %) (dependencies-url namespace)))))))



(defn classpath-files []
  (map file-str (clojure.string/split (classpath) #":")))

;;Every file that comes back from dependencies-file is also on the classpath.
;;In order to recreate the right project structure, we need to copy the files 
;;with the appropiate classpath nesting.

(defn bifurcate
  "split a collection between f and not f"
  [f coll]
  (list
   (filter f coll)
   (filter (comp not f) coll)))

(defn jar? [#^java.io.File f]
  (re-matches  #".*\.jar$" (.getCanonicalPath f)))

(defn contains-file? [#^java.io.File parent #^java.io.File child]
  (let [new-child (.getParentFile child)]
    (cond (nil? new-child) false
	  (= new-child parent) true
	  true (recur parent new-child))))

(defn destination [base-path-list #^java.io.File current-file #^java.io.File base-destination]
  (let [base-path (last (filter #(contains-file? % current-file) base-path-list))]
    (file-str
     (.replaceFirst (.getCanonicalPath current-file)
		    (.getCanonicalPath base-path)
		    (.getCanonicalPath base-destination)))))

(defn export-dependencies
  ([namespace #^java.io.File target src* lib*]
     (let [[jars sources] (bifurcate jar? (dependencies-file namespace))
	   jars (if *nailgun*
		  (conj jars (file-str "~/roBin/nailgun-0.7.1/nailgun-0.7.1.jar"))
		  jars)
	   
	   lib (file-str (str (.getCanonicalPath target) File/separatorChar lib*))
	   src (file-str (str (.getCanonicalPath target) File/separatorChar src*))]
       (if *nailgun*
	 (do
	   (FileUtils/copyFileToDirectory (file-str "~/roBin/nailgun-0.7.1/ng") lib)
	   (sw (str "chmod +x " (.getCanonicalPath target) File/separatorChar lib* "/ng"))))
       (dorun (map #(FileUtils/copyFileToDirectory % lib) jars))
       (dorun (map #(FileUtils/copyFile % (destination (classpath-files) % src)) sources))))
  
  ([namespace #^java.io.File target]
     (export-dependencies namespace target "src" "lib")))

(defn run-script-text
  ([namespace]
     (run-script-text namespace "src" "lib"))
  ([namespace src lib] 
     (if *nailgun*
       (str
	
	"#!/usr/bin/perl

if (`./"lib"/ng ng-alias 2>&1 1>/dev/null` eq \"connect: Connection refused\\n\"){
	
	$nailgun_init = 
	\"java -Xmn500M -Xms2000M -Xmx2000M \" . 
	\"-classpath ./"src"/:"lib"/*\" . \" -server \". 
	\"com.martiansoftware.nailgun.NGServer \". 
	\" >/dev/null 2>/dev/null &\";
	`$nailgun_init`;
	}\n\n"
	
	
	"while (`./"lib"/ng ng-alias 2>&1 1>/dev/null` eq \"connect: Connection refused\\n\"){
    }\n\n"
	
	"$command = "
	
	
	"\"./"lib"/ng clojure.main "
	"./"src"/"
	(.replace (.replace (str *ns*) \. File/separatorChar) \- \_)
	".clj "
	" @ARGV\";\n"
	
	"print `$command`;\n")
       (str
	"#!/bin/bash\n"
	"java -Xmn500M -Xms2000M -Xmx2000M -server -cp ./"lib"/*:./"src" clojure.main "
	"./"src"/"
	(.replace (.replace (str *ns*) \. File/separatorChar) \- \_)
	".clj "
	" $@\n"))))



(defn make-run-script
  ([namespace base-dir name]
     (make-run-script namespace base-dir name "src" "lib"))
  ([namespace base-dir name src lib]
  (let [w (clojure.java.io/writer
	   (str (.getCanonicalPath base-dir) File/separatorChar name))]
    (.write w (run-script-text namespace src lib))
    (.close w))
  (let [f (file-str  (str (.getCanonicalPath base-dir) File/separatorChar name))]
    (.setExecutable f true))))


(defn bzip-folder [#^java.io.File destination #^java.io.File directory]
  (apply (partial sw "tar" "-cvjf" (.getCanonicalPath destination))

	  (concat
	   (rest
	    (map #(.replaceFirst
		   %
		   (str (.getCanonicalPath directory) File/separatorChar ) "")
		 (map str (file-seq directory))))
	    [:dir (.getCanonicalPath directory)])))


(defn bzip-export-files
  ([directory to name postfix]
     (bzip-export-files directory to name postfix "src" "lib"))

  ([directory to name postfix src lib]
     (apply (partial sw "tar" "-cvjf"
		     (str (.getCanonicalPath to) File/separatorChar (str name postfix)))
	    (concat
	     [name src lib]
	     [:dir (.getCanonicalPath directory)]))))

(defn export-archive
  ([name]
     (export-archive *ns* name))

  ([namespace name nailgun?]
     (binding [*nailgun* nailgun?]
       (export-archive namespace name)))
  
  ([namespace name]
     (FileUtils/forceMkdir standard-base-directory)
     (FileUtils/forceDelete standard-base-directory)
     (FileUtils/forceMkdir standard-base-directory)
     (let [new-dir (file-str (str
			      (.getCanonicalPath standard-base-directory)
			      (File/separatorChar)
			      name))]
       (export-dependencies namespace new-dir)
       (make-run-script namespace new-dir name)
       (println (str name "-clojure.tar.bz2"))
       (println name)
       (bzip-export-files new-dir (file-str "~/coderloop/") name "-clojure.tar.bz2")
       (FileUtils/copyFileToDirectory
	(file-str (str "~/coderloop/" name "-clojure.tar.bz2"))
	(file-str "~/coderloop-test")))))
